@@ -226,6 +226,7 @@ class TaskRunner(ABC):
             args=(unit, agent),
             name=f"Unit-thread-{unit.db_id}",
         )
+        agent.update_status(AgentState.STATUS_IN_TASK)
         self.running_units[unit.db_id] = RunningUnit(
             unit=unit,
             agent=agent,
@@ -280,6 +281,9 @@ class TaskRunner(ABC):
                     if unit_agent is not None and unit_agent.db_id == agent.db_id:
                         logger.debug(f"Clearing {agent} from {unit} due to {e}")
                         unit.clear_assigned_agent()
+                    if agent.get_status() not in AgentState.complete():
+                        # Absent agents at this stage should be disconnected
+                        agent.update_status(AgentState.STATUS_DISCONNECT)
                 self.cleanup_unit(unit)
             except Exception as e:
                 logger.exception(f"Unhandled exception in unit {unit}", exc_info=True)
@@ -299,6 +303,7 @@ class TaskRunner(ABC):
 
             self._cleanup_special_units(unit, agent)
             self.task_run.clear_reservation(unit)
+            agent.hide_state()
 
     def execute_assignment(
         self,
@@ -354,6 +359,8 @@ class TaskRunner(ABC):
                         # Must expire the disconnected unit so that
                         # new workers aren't shown it
                         agent.get_unit().expire()
+                        if agent.get_status() not in AgentState.complete():
+                            agent.update_status(AgentState.STATUS_DISCONNECT)
                 self.cleanup_assignment(assignment)
             except Exception as e:
                 logger.exception(
@@ -380,6 +387,9 @@ class TaskRunner(ABC):
             for unit in assignment.get_units():
                 task_run.clear_reservation(unit)
 
+            for agent in agents:
+                agent.hide_state()
+
     @staticmethod
     def get_data_for_assignment(assignment: "Assignment") -> "InitializationData":
         """
