@@ -12,8 +12,13 @@ import json
 from typing import Dict, Any
 
 
-# TODO Hydrize
+# TODO(CLEAN) Hydrize
 def main():
+    confirm = input(
+        "This is going to completely delete all resources for your EC2 architect, including the VPC, routes, and fallback server. Are you sure you want to do this?[yes/no]\n>> "
+    )
+    if confirm != "yes":
+        return
     iam_role_name = input("Please enter local profile name for IAM role\n>> ")
     ec2_helpers.cleanup_fallback_server(iam_role_name)
 
