@@ -15,6 +15,7 @@ from mephisto.abstractions.blueprints.mixins.onboarding_required import (
     OnboardingRequiredArgs,
 )
 from dataclasses import dataclass, field
+
 from mephisto.data_model.assignment import InitializationData
 from mephisto.abstractions.blueprints.parlai_chat.parlai_chat_agent_state import (
     ParlAIChatAgentState,
@@ -29,7 +30,6 @@ from mephisto.operations.registry import register_mephisto_abstraction
 from omegaconf import DictConfig, MISSING
 
 import os
-import time
 import csv
 import sys
 import json
@@ -40,13 +40,12 @@ from typing import ClassVar, List, Type, Any, Dict, Iterable, Optional, TYPE_CHE
 
 if TYPE_CHECKING:
     from mephisto.data_model.worker import Worker
-    from mephisto.data_model.agent import Agent, OnboardingAgent
+    from mephisto.data_model.agent import OnboardingAgent
     from mephisto.data_model.task_run import TaskRun
     from mephisto.abstractions.blueprint import AgentState, TaskRunner, TaskBuilder
-    from mephisto.data_model.assignment import Assignment
-    from argparse import _ArgumentGroup as ArgumentGroup
 
-BLUEPRINT_TYPE = "parlai_chat"
+
+BLUEPRINT_TYPE_PARLAI_CHAT = "parlai_chat"
 
 
 MISSING_SOMETHING_TEXT = (
@@ -68,7 +67,7 @@ class SharedParlAITaskState(OnboardingSharedState, SharedTaskState):
 
 @dataclass
 class ParlAIChatBlueprintArgs(OnboardingRequiredArgs, BlueprintArgs):
-    _blueprint_type: str = BLUEPRINT_TYPE
+    _blueprint_type: str = BLUEPRINT_TYPE_PARLAI_CHAT
     _group: str = field(
         default="ParlAIChatBlueprint",
         metadata={
@@ -139,12 +138,7 @@ class ParlAIChatBlueprint(OnboardingRequired, Blueprint):
     TaskRunnerClass: ClassVar[Type["TaskRunner"]] = ParlAIChatTaskRunner
     ArgsClass = ParlAIChatBlueprintArgs
     SharedStateClass = SharedParlAITaskState
-    supported_architects: ClassVar[List[str]] = [
-        "mock",
-        "heroku",
-        "local",
-    ]  # TODO update?
-    BLUEPRINT_TYPE = BLUEPRINT_TYPE
+    BLUEPRINT_TYPE = BLUEPRINT_TYPE_PARLAI_CHAT
 
     def __init__(
         self,
@@ -154,7 +148,6 @@ class ParlAIChatBlueprint(OnboardingRequired, Blueprint):
     ):
         super().__init__(task_run, args, shared_state)
         self._initialization_data_dicts: List[Dict[str, Any]] = []
-
         if args.blueprint.get("context_csv", None) is not None:
             csv_file = os.path.expanduser(args.blueprint.context_csv)
             with open(csv_file, "r", encoding="utf-8-sig") as csv_fp:
