@@ -5,14 +5,17 @@
 # LICENSE file in the root directory of this source tree.
 
 from __future__ import annotations
+import csv
+from genericpath import exists
 
 import os
+from pathlib import Path
 import threading
 from queue import Queue
 from uuid import uuid4
 from prometheus_client import Gauge  # type: ignore
 
-from abc import ABC, abstractmethod, abstractstaticmethod
+from abc import ABC, abstractmethod
 from mephisto.abstractions.blueprint import AgentState
 from mephisto.data_model.worker import Worker
 from mephisto.data_model._db_backed_meta import (
@@ -26,7 +29,14 @@ from mephisto.data_model.exceptions import (
     AgentShutdownError,
 )
 
-from typing import List, Optional, Tuple, Mapping, Dict, Any, cast, TYPE_CHECKING
+from typing import Optional, Mapping, Dict, Any, cast, TYPE_CHECKING
+
+try:
+    from detoxify import Detoxify
+
+    DETOXIFY_INSTALLED = True
+except ImportError:
+    DETOXIFY_INSTALLED = False
 
 if TYPE_CHECKING:
     from mephisto.data_model.unit import Unit
@@ -57,8 +67,304 @@ ACTIVE_WORKERS = Gauge(
 )
 
 
-# TODO(CLEAN) can probably refactor out some kind of AgentBase
-class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
+class _AgentBase(ABC):
+    """
+    This class contains the shared logic between Agents and OnboardingAgents,
+    and should be extended by any class acting like an Agent in the
+    mephisto data model.
+    """
+
+    def __init__(
+        self,
+        db: "MephistoDB",
+        db_id: str,
+        row: Optional[Mapping[str, Any]] = None,
+        _used_new_call: bool = False,
+    ):
+        if not _used_new_call:
+            raise AssertionError(
+                "Direct Agent and data model access via ...Agent(db, id) was "
+                "now deprecated in favor of calling Agent.get(db, id). "
+            )
+        self.db: "MephistoDB" = db
+        assert row is not None, f"AbstractAgent requires a row be provided"
+
+        # Loading of database attributes
+        self.db_id = db_id
+        self.db_status: str = row["status"]
+        self.worker_id: str = row["worker_id"]
+        self.task_type: str = row["task_type"]
+        self.task_run_id: str = row["task_run_id"]
+        self.task_id: str = row["task_id"]
+
+        # Deferred loading of related entities
+        self._worker: Optional["Worker"] = None
+        self._task_run: Optional["TaskRun"] = None
+        self._task: Optional["Task"] = None
+
+        # Related entity set by a live run
+        self._associated_live_run: Optional["LiveTaskRun"] = None
+
+        # Local state for live agents
+        self.pending_actions: "Queue[Dict[str, Any]]" = Queue()
+        self.has_live_update = threading.Event()
+        self.has_live_update.clear()
+        self.did_submit = threading.Event()
+        self.is_shutdown = False
+
+        # Follow-up initialization is deferred
+        self._state = None  # type: ignore
+
+    @property
+    def state(self) -> "AgentState":
+        if self._state is None:
+            self._state = AgentState(self)  # type: ignore
+        return cast("AgentState", self._state)
+
+    def hide_state(self) -> None:
+        """Clean up the state such that the GC may remove it"""
+        self._state = None
+
+    def set_live_run(self, live_run: "LiveTaskRun") -> None:
+        """Set an associated live run for this agent"""
+        self._associated_live_run = live_run
+
+    def get_live_run(self) -> "LiveTaskRun":
+        """Return the associated live run for this agent. Throw if not set"""
+        if self._associated_live_run is None:
+            raise AssertionError(
+                "Should not be getting the live run, not set for given agent"
+            )
+        return self._associated_live_run
+
+    def agent_in_active_run(self) -> bool:
+        """
+        Returns whether the given agent is in an active LiveTaskRun
+        and can have status updates
+        """
+        if self._associated_live_run is None:
+            return False  # No live run
+        if self._associated_live_run.client_io.is_shutdown:
+            return False  # Live run, but is shutdown
+        return True
+
+    def get_worker(self) -> Worker:
+        """
+        Return the worker that is using this agent for a task
+        """
+        if self._worker is None:
+            self._worker = Worker.get(self.db, self.worker_id)
+        return self._worker
+
+    def get_task_run(self) -> "TaskRun":
+        """Return the TaskRun this agent is working within"""
+        if self._task_run is None:
+            from mephisto.data_model.task_run import TaskRun
+
+            self._task_run = TaskRun.get(self.db, self.task_run_id)
+        return self._task_run
+
+    def get_task(self) -> "Task":
+        """Return the Task this agent is working within"""
+        if self._task is None:
+            if self._task_run is not None:
+                self._task = self._task_run.get_task()
+            else:
+                from mephisto.data_model.task import Task
+
+                self._task = Task.get(self.db, self.task_id)
+        return self._task
+
+    def observe(self, live_update: "Dict[str, Any]") -> None:
+        """
+        Pass the observed information to the AgentState, then
+        queue the information to be pushed to the user
+        """
+        if live_update.get("update_id") is None:
+            live_update["update_id"] = str(uuid4())
+        self.state.update_data(live_update)
+
+        if self.agent_in_active_run():
+            live_run = self.get_live_run()
+            live_run.client_io.send_live_update(self.get_agent_id(), live_update)
+
+    def get_live_update(
+        self, timeout: Optional[int] = None
+    ) -> Optional[Dict[str, Any]]:
+        """
+        Request information from the Agent's frontend. If non-blocking,
+        (timeout is None) should return None if no actions are ready
+        to be returned.
+        """
+        if self.pending_actions.empty():
+            if timeout is None or timeout == 0:
+                return None
+            self.has_live_update.wait(timeout)
+
+        if self.pending_actions.empty():
+            if self.is_shutdown:
+                raise AgentShutdownError(self.db_id)
+            # various disconnect cases
+            status = self.get_status()
+            if status == AgentState.STATUS_DISCONNECT:
+                raise AgentDisconnectedError(self.db_id)
+            elif status == AgentState.STATUS_RETURNED:
+                raise AgentReturnedError(self.db_id)
+            self.update_status(AgentState.STATUS_TIMEOUT)
+            raise AgentTimeoutError(timeout, self.db_id)
+        assert (
+            not self.pending_actions.empty()
+        ), "has_live_update released without an action!"
+
+        act = self.pending_actions.get()
+
+        if self.pending_actions.empty():
+            self.has_live_update.clear()
+        self.state.update_data(act)
+        return act
+
+    def act(self, timeout: Optional[int] = None) -> Optional[Dict[str, Any]]:
+        """
+        Request information from the Agent's frontend. If non-blocking,
+        (timeout is None) should return None if no actions are ready
+        to be returned.
+        """
+        warn_once(
+            "As of Mephisto 1.0 Agent.act is being deprecated in favor of Agent.get_live_update. "
+            "This functionality will no longer work in 1.1"
+        )
+        return self.get_live_update(timeout)
+
+    def await_submit(self, timeout: Optional[int] = None) -> bool:
+        """
+        Blocking wait for this agent to submit their task
+        If timeout is provided and exceeded, raises AgentTimeoutError
+        """
+        if timeout is not None:
+            # Handle disconnect possibilities first
+            status = self.get_status()
+            if status == AgentState.STATUS_DISCONNECT:
+                raise AgentDisconnectedError(self.db_id)
+            elif status == AgentState.STATUS_RETURNED:
+                raise AgentReturnedError(self.db_id)
+            elif status == AgentState.STATUS_TIMEOUT:
+                raise AgentTimeoutError(timeout, self.db_id)
+            # Wait for the status change
+            self.did_submit.wait(timeout=timeout)
+            if not self.did_submit.is_set():
+                # If released without submit, raise timeout
+                raise AgentTimeoutError(timeout, self.db_id)
+        return self.did_submit.is_set()
+
+    def handle_submit(self, submit_data: Dict[str, Any]) -> None:
+        """Handle final submission for an onboarding agent, with the given data"""
+        self.did_submit.set()
+        self.state.update_submit(submit_data)
+
+    def handle_metadata_submit(self, data: Dict[str, Any]) -> None:
+        """Handles the submission of metadata (as of now that is tips and feedback)"""
+
+        if "tips" in data:
+            """Handles the submission of a tip"""
+            assert (
+                hasattr(self.state.metadata, "tips") == True
+            ), "The {property_name} field must exist in _AgentStateMetadata. Go into _AgentStateMetadata and add the {property_name} field".format(
+                property_name="tips"
+            )
+            new_tip_header = data["tips"]["header"]
+            new_tip_text = data["tips"]["text"]
+            copy_of_tips = None
+            tip_to_add = {
+                "id": str(uuid4()),
+                "header": new_tip_header,
+                "text": new_tip_text,
+                "accepted": False,
+            }
+            if self.state.metadata.tips is None:
+                self.state.update_metadata(
+                    property_name="tips", property_value=[tip_to_add]
+                )
+            else:
+                copy_of_tips = self.state.metadata.tips.copy()
+                copy_of_tips.append(tip_to_add)
+                self.state.update_metadata(
+                    property_name="tips", property_value=copy_of_tips
+                )
+
+        elif "feedback" in data:
+            questions_and_answers = data["feedback"]["data"]
+            for question_obj in questions_and_answers:
+                new_feedback_text = question_obj["text"]
+                new_feedback_toxicity = (
+                    Detoxify("original").predict(new_feedback_text)["toxicity"]
+                    if DETOXIFY_INSTALLED == True
+                    else None
+                )
+                feedback_to_add = {
+                    "id": str(uuid4()),
+                    "question": question_obj["question"],
+                    "text": new_feedback_text,
+                    "reviewed": False,
+                    "toxicity": None
+                    if new_feedback_toxicity is None
+                    else str(new_feedback_toxicity),
+                }
+                if self.state.metadata.feedback is None:
+                    self.state.update_metadata(
+                        property_name="feedback", property_value=[feedback_to_add]
+                    )
+                else:
+                    copy_of_feedback = self.state.metadata.feedback.copy()
+                    copy_of_feedback.append(feedback_to_add)
+                    self.state.update_metadata(
+                        property_name="feedback", property_value=copy_of_feedback
+                    )
+
+    def shutdown(self) -> None:
+        """
+        Force the given agent to end any polling threads and throw an AgentShutdownError
+        from any acts called on it, ensuring tasks using this agent can be cleaned up.
+        """
+        logger.debug(f"{self} is shutting down")
+        self.has_live_update.set()
+        self.did_submit.set()
+        self.is_shutdown = True
+
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({self.db_id}, {self.db_status})"
+
+    @abstractmethod
+    def get_agent_id(self) -> str:
+        """
+        Return the ID that should be used to refer to this agent
+        in the Mephisto IO API
+        """
+        raise NotImplementedError
+
+    @abstractmethod
+    def get_data_dir(self) -> str:
+        """
+        Return the directory to be storing any agent state for
+        this agent into
+        """
+        raise NotImplementedError
+
+    @abstractmethod
+    def update_status(self, new_status: str) -> None:
+        """Update the database status of this agent, and
+        possibly send a message to the frontend agent informing
+        them of this update"""
+        raise NotImplementedError
+
+    @abstractmethod
+    def get_status(self) -> str:
+        """Get the status of this agent in their work on their unit"""
+        raise NotImplementedError
+
+
+class Agent(
+    _AgentBase, MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta
+):
     """
     This class encompasses a worker as they are working on an individual assignment.
     It maintains details for the current task at hand such as start and end time,
@@ -81,39 +387,19 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
         if row is None:
             row = db.get_agent(db_id)
         assert row is not None, f"Given db_id {db_id} did not exist in given db"
-        self.db_id: str = row["agent_id"]
-        self.db_status: str = row["status"]
-        self.worker_id: str = row["worker_id"]
+
+        # Cover basic initialization
+        _AgentBase.__init__(self, db, db_id, row=row, _used_new_call=_used_new_call)
+
+        # Agent-specific row fields
         self.unit_id: str = row["unit_id"]
-        self.task_type: str = row["task_type"]
         self.provider_type: str = row["provider_type"]
-        self.pending_actions: "Queue[Dict[str, Any]]" = Queue()
-        self.has_live_update = threading.Event()
-        self.has_live_update.clear()
         self.assignment_id: str = row["assignment_id"]
-        self.task_run_id: str = row["task_run_id"]
-        self.task_id: str = row["task_id"]
-        self.did_submit = threading.Event()
-        self.is_shutdown = False
 
         # Deferred loading of related entities
         self._worker: Optional["Worker"] = None
         self._unit: Optional["Unit"] = None
         self._assignment: Optional["Assignment"] = None
-        self._task_run: Optional["TaskRun"] = None
-        self._task: Optional["Task"] = None
-
-        # Related entity set by a live run
-        self._associated_live_run: Optional["LiveTaskRun"] = None
-
-        # Follow-up initialization is deferred
-        self._state = None  # type: ignore
-
-    @property
-    def state(self) -> "AgentState":
-        if self._state is None:
-            self._state = AgentState(self)  # type: ignore
-        return cast("AgentState", self._state)
 
     def __new__(
         cls,
@@ -145,30 +431,10 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
             # We are constructing another instance directly
             return super().__new__(cls)
 
-    def set_live_run(self, live_run: "LiveTaskRun") -> None:
-        """Set an associated live run for this agent"""
-        self._associated_live_run = live_run
-
-    def get_live_run(self) -> "LiveTaskRun":
-        """Return the associated live run for this agent. Throw if not set"""
-        if self._associated_live_run is None:
-            raise AssertionError(
-                "Should not be getting the live run, not set for given agent"
-            )
-        return self._associated_live_run
-
     def get_agent_id(self) -> str:
         """Return this agent's id"""
         return self.db_id
 
-    def get_worker(self) -> Worker:
-        """
-        Return the worker that is using this agent for a task
-        """
-        if self._worker is None:
-            self._worker = Worker.get(self.db, self.worker_id)
-        return self._worker
-
     def get_unit(self) -> "Unit":
         """
         Return the Unit that this agent is working on.
@@ -190,34 +456,6 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
                 self._assignment = Assignment.get(self.db, self.assignment_id)
         return self._assignment
 
-    def get_task_run(self) -> "TaskRun":
-        """Return the TaskRun this agent is working within"""
-        if self._task_run is None:
-            if self._unit is not None:
-                self._task_run = self._unit.get_task_run()
-            elif self._assignment is not None:
-                self._task_run = self._assignment.get_task_run()
-            else:
-                from mephisto.data_model.task_run import TaskRun
-
-                self._task_run = TaskRun.get(self.db, self.task_run_id)
-        return self._task_run
-
-    def get_task(self) -> "Task":
-        """Return the Task this agent is working within"""
-        if self._task is None:
-            if self._unit is not None:
-                self._task = self._unit.get_task()
-            elif self._assignment is not None:
-                self._task = self._assignment.get_task()
-            elif self._task_run is not None:
-                self._task = self._task_run.get_task()
-            else:
-                from mephisto.data_model.task import Task
-
-                self._task = Task.get(self.db, self.task_id)
-        return self._task
-
     def get_data_dir(self) -> str:
         """
         Return the directory to be storing any agent state for
@@ -239,7 +477,7 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
         old_status = self.db_status
         self.db.update_agent(self.db_id, status=new_status)
         self.db_status = new_status
-        if self._associated_live_run is not None:
+        if self.agent_in_active_run():
             live_run = self.get_live_run()
             live_run.loop_wrap.execute_coro(
                 live_run.worker_pool.push_status_update(self)
@@ -313,77 +551,6 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
         agent._unit = unit
         return agent
 
-    def observe(self, live_update: "Dict[str, Any]") -> None:
-        """
-        Pass the observed information to the AgentState, then
-        queue the information to be pushed to the user
-        """
-        if live_update.get("update_id") is None:
-            live_update["update_id"] = str(uuid4())
-        self.state.update_data(live_update)
-
-        if self._associated_live_run is not None:
-            live_run = self.get_live_run()
-            live_run.client_io.send_live_update(self.get_agent_id(), live_update)
-
-    def get_live_update(
-        self, timeout: Optional[int] = None
-    ) -> Optional[Dict[str, Any]]:
-        """
-        Request information from the Agent's frontend. If non-blocking,
-        (timeout is None) should return None if no actions are ready
-        to be returned.
-        """
-        if self.pending_actions.empty():
-            if timeout is None or timeout == 0:
-                return None
-            self.has_live_update.wait(timeout)
-
-        if self.pending_actions.empty():
-            if self.is_shutdown:
-                raise AgentShutdownError(self.db_id)
-            # various disconnect cases
-            status = self.get_status()
-            if status == AgentState.STATUS_DISCONNECT:
-                raise AgentDisconnectedError(self.db_id)
-            elif status == AgentState.STATUS_RETURNED:
-                raise AgentReturnedError(self.db_id)
-            self.update_status(AgentState.STATUS_TIMEOUT)
-            raise AgentTimeoutError(timeout, self.db_id)
-        assert (
-            not self.pending_actions.empty()
-        ), "has_live_update released without an action!"
-
-        act = self.pending_actions.get()
-
-        if self.pending_actions.empty():
-            self.has_live_update.clear()
-        self.state.update_data(act)
-        return act
-
-    def act(self, timeout: Optional[int] = None) -> Optional[Dict[str, Any]]:
-        """
-        Request information from the Agent's frontend. If non-blocking,
-        (timeout is None) should return None if no actions are ready
-        to be returned.
-        """
-        warn_once(
-            "As of Mephisto 1.0 Agent.act is being deprecated in favor of Agent.get_live_update. "
-            "This functionality will no longer work in 1.1"
-        )
-        return self.get_live_update(timeout)
-
-    def await_submit(self, timeout: Optional[int] = None) -> bool:
-        """Blocking wait for this agent to submit their task"""
-        if timeout is not None:
-            self.did_submit.wait(timeout=timeout)
-        return self.did_submit.is_set()
-
-    def handle_submit(self, submit_data: Dict[str, Any]) -> None:
-        """Handle final submission for an onboarding agent, with the given data"""
-        self.did_submit.set()
-        self.state.update_submit(submit_data)
-
     def get_status(self) -> str:
         """Get the status of this agent in their work on their unit"""
         if self.db_status not in AgentState.complete():
@@ -395,7 +562,7 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
                 ]:
                     # Disconnect statuses should free any pending acts
                     self.has_live_update.set()
-                if self._associated_live_run is not None:
+                if self.agent_in_active_run():
                     live_run = self.get_live_run()
                     live_run.loop_wrap.execute_coro(
                         live_run.worker_pool.push_status_update(self)
@@ -403,18 +570,6 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
             self.db_status = row["status"]
         return self.db_status
 
-    def shutdown(self) -> None:
-        """
-        Force the given agent to end any polling threads and throw an AgentShutdownError
-        from any acts called on it, ensuring tasks using this agent can be cleaned up.
-        """
-        logger.debug(f"{self} is shutting down")
-        self.has_live_update.set()
-        self.is_shutdown = True
-
-    def __repr__(self) -> str:
-        return f"{self.__class__.__name__}({self.db_id}, {self.db_status})"
-
     # Children classes should implement the following methods
 
     def approve_work(self) -> None:
@@ -457,7 +612,7 @@ class Agent(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta):
 
 
 class OnboardingAgent(
-    MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta
+    _AgentBase, MephistoDataModelComponentMixin, metaclass=MephistoDBBackedABCMeta
 ):
     """
     Onboarding agents are a special extension of agents used
@@ -486,49 +641,15 @@ class OnboardingAgent(
                 "Direct OnboardingAgent and data model access via OnboardingAgent(db, id) is "
                 "now deprecated in favor of calling OnboardingAgent.get(db, id). "
             )
-        self.db: "MephistoDB" = db
         if row is None:
             row = db.get_onboarding_agent(db_id)
         assert row is not None, f"Given db_id {db_id} did not exist in given db"
-        self.db_id: str = row["onboarding_agent_id"]
-        self.db_status: str = row["status"]
-        self.worker_id: str = row["worker_id"]
-        self.task_type: str = row["task_type"]
-        self.pending_actions: "Queue[Dict[str, Any]]" = Queue()
-        self.has_live_update = threading.Event()
-        self.has_live_update.clear()
-        self.task_run_id: str = row["task_run_id"]
-        self.task_id: str = row["task_id"]
-        self.did_submit = threading.Event()
-        self.is_shutdown = False
-
-        # Deferred loading of related entities
-        self._worker: Optional["Worker"] = None
-        self._task_run: Optional["TaskRun"] = None
-        self._task: Optional["Task"] = None
-
-        # Related entity set by a live run
-        self._associated_live_run: Optional["LiveTaskRun"] = None
-
-        # Follow-up initialization
-        self.state = AgentState(self)  # type: ignore
+        _AgentBase.__init__(self, db, db_id, row=row, _used_new_call=_used_new_call)
 
     def get_agent_id(self) -> str:
         """Return an id to use for onboarding agent requests"""
         return f"{self.DISPLAY_PREFIX}{self.db_id}"
 
-    def set_live_run(self, live_run: "LiveTaskRun") -> None:
-        """Set an associated live run for this agent"""
-        self._associated_live_run = live_run
-
-    def get_live_run(self) -> "LiveTaskRun":
-        """Return the associated live run for this agent. Throw if not set"""
-        if self._associated_live_run is None:
-            raise AssertionError(
-                "Should not be getting the live run, not set for given agent"
-            )
-        return self._associated_live_run
-
     @classmethod
     def is_onboarding_id(cls, agent_id: str) -> bool:
         """return if the given id is for an onboarding agent"""
@@ -542,33 +663,6 @@ class OnboardingAgent(
         ), f"Provided id {agent_id} is not an onboarding_id"
         return agent_id[len(cls.DISPLAY_PREFIX) :]
 
-    def get_worker(self) -> Worker:
-        """
-        Return the worker that is using this agent for a task
-        """
-        if self._worker is None:
-            self._worker = Worker.get(self.db, self.worker_id)
-        return self._worker
-
-    def get_task_run(self) -> "TaskRun":
-        """Return the TaskRun this agent is working within"""
-        if self._task_run is None:
-            from mephisto.data_model.task_run import TaskRun
-
-            self._task_run = TaskRun.get(self.db, self.task_run_id)
-        return self._task_run
-
-    def get_task(self) -> "Task":
-        """Return the Task this agent is working within"""
-        if self._task is None:
-            if self._task_run is not None:
-                self._task = self._task_run.get_task()
-            else:
-                from mephisto.data_model.task import Task
-
-                self._task = Task.get(self.db, self.task_id)
-        return self._task
-
     def get_data_dir(self) -> str:
         """
         Return the directory to be storing any agent state for
@@ -591,7 +685,7 @@ class OnboardingAgent(
         old_status = self.db_status
         self.db.update_onboarding_agent(self.db_id, status=new_status)
         self.db_status = new_status
-        if self._associated_live_run is not None:
+        if self.agent_in_active_run():
             if new_status not in [
                 AgentState.STATUS_APPROVED,
                 AgentState.STATUS_REJECTED,
@@ -616,77 +710,6 @@ class OnboardingAgent(
                 worker_id=self.worker_id, agent_type="onboarding"
             ).dec()
 
-    def observe(self, live_update: "Dict[str, Any]") -> None:
-        """
-        Pass the observed information to the AgentState, then
-        queue the information to be pushed to the user
-        """
-        if live_update.get("update_id") is None:
-            live_update["update_id"] = str(uuid4())
-        self.state.update_data(live_update)
-
-        if self._associated_live_run is not None:
-            live_run = self.get_live_run()
-            live_run.client_io.send_live_update(self.get_agent_id(), live_update)
-
-    def get_live_update(
-        self, timeout: Optional[int] = None
-    ) -> Optional[Dict[str, Any]]:
-        """
-        Request information from the Agent's frontend. If non-blocking,
-        (timeout is None) should return None if no actions are ready
-        to be returned.
-        """
-        if self.pending_actions.empty():
-            if timeout is None or timeout == 0:
-                return None
-            self.has_live_update.wait(timeout)
-
-        if self.pending_actions.empty():
-            # various disconnect cases
-            if self.is_shutdown:
-                raise AgentShutdownError(self.db_id)
-            status = self.get_status()
-            if status == AgentState.STATUS_DISCONNECT:
-                raise AgentDisconnectedError(self.db_id)
-            elif status == AgentState.STATUS_RETURNED:
-                raise AgentReturnedError(self.db_id)
-            self.update_status(AgentState.STATUS_TIMEOUT)
-            raise AgentTimeoutError(timeout, self.db_id)
-        assert (
-            not self.pending_actions.empty()
-        ), "has_live_update released without an action!"
-
-        act = self.pending_actions.get()
-
-        if self.pending_actions.empty():
-            self.has_live_update.clear()
-        self.state.update_data(act)
-        return act
-
-    def act(self, timeout: Optional[int] = None) -> Optional[Dict[str, Any]]:
-        """
-        Request information from the Agent's frontend. If non-blocking,
-        (timeout is None) should return None if no actions are ready
-        to be returned.
-        """
-        warn_once(
-            "As of Mephisto 1.0 Agent.act is being deprecated in favor of Agent.get_live_update. "
-            "This functionality will no longer work in 1.1"
-        )
-        return self.get_live_update(timeout)
-
-    def await_submit(self, timeout: Optional[int] = None) -> bool:
-        """Blocking wait for this agent to submit their task"""
-        if timeout is not None:
-            self.did_submit.wait(timeout=timeout)
-        return self.did_submit.is_set()
-
-    def handle_submit(self, submit_data: Dict[str, Any]) -> None:
-        """Handle final submission for an onboarding agent, with the given data"""
-        self.did_submit.set()
-        self.state.update_submit(submit_data)
-
     def get_status(self) -> str:
         """Get the status of this agent in their work on their unit"""
         if self.db_status not in AgentState.complete():
@@ -702,7 +725,7 @@ class OnboardingAgent(
                     AgentState.STATUS_APPROVED,
                     AgentState.STATUS_REJECTED,
                 ]:
-                    if self._associated_live_run is not None:
+                    if self.agent_in_active_run():
                         live_run = self.get_live_run()
                         live_run.loop_wrap.execute_coro(
                             live_run.worker_pool.push_status_update(self)
@@ -710,15 +733,6 @@ class OnboardingAgent(
             self.db_status = row["status"]
         return self.db_status
 
-    def shutdown(self) -> None:
-        """
-        Force the given agent to end any polling threads and throw an AgentShutdownError
-        from any acts called on it, ensuring tasks using this agent can be cleaned up.
-        """
-        logger.debug(f"{self} is shutting down")
-        self.has_live_update.set()
-        self.is_shutdown = True
-
     @staticmethod
     def new(db: "MephistoDB", worker: Worker, task_run: "TaskRun") -> "OnboardingAgent":
         """
@@ -734,6 +748,3 @@ class OnboardingAgent(
         ACTIVE_WORKERS.labels(worker_id=worker.db_id, agent_type="onboarding").inc()
         logger.debug(f"Registered new {a} for worker {worker}.")
         return a
-
-    def __repr__(self) -> str:
-        return f"{self.__class__.__name__}({self.db_id})"
