@@ -74,6 +74,10 @@ class TaskRunArgs:
             "required": True,
         },
     )
+    task_lifetime_in_seconds: int = field(
+        default=60 * 60 * 24 * 31,
+        metadata={"help": "The time that the task will last for before expiring"},
+    )
     assignment_duration_in_seconds: int = field(
         default=30 * 60,
         metadata={"help": "Time that workers have to work on your task once accepted."},
@@ -113,6 +117,27 @@ class TaskRunArgs:
         },
     )
 
+    post_install_script: str = field(
+        default="",
+        metadata={
+            "help": (
+                "The name of a shell script in your webapp directory that will run right after npm install and before npm build."
+                "This can be useful for local package development where you would want to link a package after installing dependencies from package.json"
+            )
+        },
+    )
+
+    force_rebuild: bool = field(
+        default=False,
+        metadata={
+            "help": (
+                "Determines if npm build should be ran every time the task is ran."
+                "By default there is an optimization that only builds the webapp when there is a change in its contents."
+                "It would make sense to set this to true when doing local package development as you want to force a rebuild after running the post_install_script."
+            )
+        },
+    )
+
     @classmethod
     def get_mock_params(cls) -> str:
         """Returns a param string with default / mock arguments to use for testing"""
@@ -191,6 +216,9 @@ class TaskRun(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedMeta):
         """
         config = self.get_task_args()
 
+        # TODO(#773) handle with temporary local qualifications to allow
+        # pushing real exclusionary qualifications after exceeding
+        # maximum_units_per_worker
         if config.allowed_concurrent != 0 or config.maximum_units_per_worker:
             current_units = self.db.find_units(
                 task_run_id=self.db_id,
@@ -236,11 +264,14 @@ class TaskRun(MephistoDataModelComponentMixin, metaclass=MephistoDBBackedMeta):
             is_self_set = map(lambda u: u.worker_id == worker.db_id, unit_set)
             if not any(is_self_set):
                 units += unit_set
+
         # Valid units must be launched and must not be special units (negative indices)
+        # Can use db_status directly rather than polling in the critical path, as in
+        # the worst case we miss the transition from an active to launched unit
         valid_units = [
             u
             for u in units
-            if u.get_status() == AssignmentState.LAUNCHED and u.unit_index >= 0
+            if u.db_status == AssignmentState.LAUNCHED and u.unit_index >= 0
         ]
         logger.debug(f"Found {len(valid_units)} available units")
 
