@@ -4,8 +4,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-
-import weakref
 import time
 import asyncio
 from queue import Queue
@@ -24,12 +22,13 @@ from mephisto.data_model.packet import (
     PACKET_TYPE_REQUEST_STATUSES,
     PACKET_TYPE_RETURN_STATUSES,
     PACKET_TYPE_ERROR,
+    PACKET_TYPE_SUBMIT_METADATA,
 )
 from mephisto.abstractions.blueprint import AgentState
-from mephisto.data_model.agent import Agent, OnboardingAgent
+from mephisto.data_model.agent import _AgentBase
 from mephisto.operations.datatypes import LiveTaskRun
 from mephisto.abstractions._subcomponents.channel import Channel, STATUS_CHECK_TIME
-from typing import Dict, Tuple, Union, Optional, List, Any, TYPE_CHECKING
+from typing import Dict, Tuple, Optional, Any, TYPE_CHECKING
 
 if TYPE_CHECKING:
     from mephisto.abstractions.database import MephistoDB
@@ -111,13 +110,10 @@ class ClientIOHandler:
         server_timestamp = packet.server_timestamp
         response_timestamp = time.time()
         if router_outgoing_timestamp is None:
-            print(packet, "no outgoing timestamp")
             router_outgoing_timestamp = server_timestamp
         if router_incoming_timestamp is None:
-            print(packet, "no incoming timestamp")
             router_incoming_timestamp = router_outgoing_timestamp
         if client_timestamp is None:
-            print(packet, "no client timestamp")
             client_timestamp = router_incoming_timestamp
         client_to_router = max(0, router_incoming_timestamp - client_timestamp)
         router_processing = max(
@@ -261,7 +257,7 @@ class ClientIOHandler:
         """Handle an action as sent from an agent, enqueuing to the agent"""
         live_run = self.get_live_run()
         agent = live_run.worker_pool.get_agent_for_id(packet.subject_id)
-        assert agent is not None, "Could not find given agent!"
+        assert agent is not None, f"Could not find given agent: {packet.subject_id}"
 
         agent.pending_actions.put(packet.data)
         agent.has_live_update.set()
@@ -272,23 +268,33 @@ class ClientIOHandler:
         agent = live_run.worker_pool.get_agent_for_id(packet.subject_id)
         assert agent is not None, "Could not find given agent!"
 
-        # If the packet is_submit, and has files, we need to
-        # process downloading those files first
-        data_files = packet.data.get("files")
-        if data_files is not None:
-            save_dir = agent.get_data_dir()
-            architect = live_run.architect
-            for f_obj in data_files:
-                # TODO(#649) this is incredibly blocking!
-                architect.download_file(f_obj["filename"], save_dir)
+        # Special handler for file downloads while we have architect access
+        # NOTE: this is a leaky abstraction at the moment - only architects
+        # know how to save files but "file saving" methods are defined by
+        # AgentStates, which don't have architect access.
+        if isinstance(packet.data, dict):
+            data_files = packet.data.get("files")
+            if data_files is not None:
+                save_dir = agent.get_data_dir()
+                architect = live_run.architect
+                for f_obj in data_files:
+                    # TODO(#649) this is incredibly blocking!
+                    architect.download_file(f_obj["filename"], save_dir)
 
         agent.handle_submit(packet.data)
 
+    def _on_submit_metadata(self, packet: Packet):
+        live_run = self.get_live_run()
+        agent = live_run.worker_pool.get_agent_for_id(packet.subject_id)
+        assert agent is not None, "Could not find given agent!"
+        agent.handle_metadata_submit(packet.data)
+
     def _on_submit_onboarding(self, packet: Packet, channel_id: str) -> None:
         """Handle the submission of onboarding data"""
         assert (
             "onboarding_data" in packet.data
         ), f"Onboarding packet {packet} submitted without data"
+        agent: Optional["_AgentBase"]
         live_run = self.get_live_run()
         onboarding_id = packet.subject_id
         if onboarding_id not in live_run.worker_pool.onboarding_agents:
@@ -296,6 +302,12 @@ class ClientIOHandler:
                 f"Onboarding agent {onboarding_id} already submitted or disconnected, "
                 f"but is calling _on_submit_onboarding again"
             )
+            # On resubmit, ensure that the client has the same status
+            agent = live_run.worker_pool.final_onboardings.get(onboarding_id)
+            if agent is not None:
+                live_run.loop_wrap.execute_coro(
+                    live_run.worker_pool.push_status_update(agent)
+                )
             return
         agent = live_run.worker_pool.get_agent_for_id(onboarding_id)
         assert agent is not None, f"Could not find given agent by id {onboarding_id}"
@@ -371,6 +383,8 @@ class ClientIOHandler:
             elif packet.type == PACKET_TYPE_SUBMIT_UNIT:
                 self._on_submit_unit(packet, channel_id)
                 self.log_metrics_for_packet(packet)
+            elif packet.type == PACKET_TYPE_SUBMIT_METADATA:
+                self._on_submit_metadata(packet)
             elif packet.type == PACKET_TYPE_MEPHISTO_BOUND_LIVE_UPDATE:
                 self._on_live_update(packet, channel_id)
                 self.log_metrics_for_packet(packet)
